import datetime
from fabric.api import env, run, get
from fabric.tasks import Task

class DumpDatabaseTask(object):
    def get_path(self, env, reason):
        mytimestamp = datetime.datetime.now().strftime('%Y-%m-%d-%H%M%S')
        reason = reason.replace(' ', '_')
        return ('%(backup_dir)s/%(db_name)s-' + mytimestamp + '-' + reason + '.sql') % env

    def get_command(self, env, file_name):
        raise NotImplementedError

    def run(self, reason='unknown', compress=False):
        file_name = self.get_path(env, reason)
        command = self.get_command(env, file_name)
        run(command)
        if compress:
            run('gzip ' + file_name)
            file_name += '.gz'
        return file_name


class DownloadDatabase(Task):
    """
    Download the database
    """

    name = 'download'

    def __init__(self, dump_task):
        self.dump_task = dump_task

    def run(self, path='tmp'):
        file_name = self.dump_task.run(reason='for_download', compress=True, nolock=True)
        get(file_name, path)


import os

from fabric.api import env, task, prompt, run, get
from fabric.contrib import files
import getpass
import json
import urllib2
import subprocess
import StringIO
import requests

@task
def upload_ssh_deploy_key():
    if not files.exists("/home/%(user)s/.ssh/id_rsa.pub" % env):
        if not files.exists("/home/%(user)s/.ssh/"):
            run("mkdir -p /home/%(user)s/.ssh/" % env)
        run("ssh-keygen -t rsa -f '/home/%(user)s/.ssh/id_rsa' -P ''" % env)

    output = StringIO.StringIO()
    get("/home/%(user)s/.ssh/id_rsa.pub" % env, output)
    output.seek(0)
    
    ssh_key = output.read()

    logged_in = False
    while not logged_in:
        try:
            default_username = subprocess.check_output(["git", "config", "--get", "github.user"]).strip()
        except Exception:
            default_username = ''
        username = prompt("Please enter your GitHub username:", default=default_username)
        password = getpass.getpass("Please enter your GitHub password: ")

        repository = env.repo.rsplit(":", 1)[-1].replace(".git", "")
        response = json.loads(requests.get("https://api.github.com/repos/%s/keys" % repository, auth=(username, password), ).content)

        if 'message' in response:
            print(response['message'])
        else:
            logged_in = True
    
    match = [x for x in response if x.get("key") in ssh_key]
    if not match:
        data = {'key': ssh_key}
        data = json.dumps(data)
        requests.post("https://api.github.com/repos/%s/keys" % repository, auth=(username, password), data=data)


import os

from fabric.api import env, task, prompt, run, put
from fabric.contrib import files
from .django_utils import django_exec
from dploi_fabric import git
from github import upload_ssh_deploy_key
from supervisor import update_config_file as supervisor_update_config_file
from nginx import update_config_file as nginx_update_config_file
import django_utils
from .utils import config
@task
def init():
    # TODO: Use utils.config (after checkout)
    if files.exists(os.path.join(env.path, 'bin')):
        print "buildout environment exists already"
        return
    upload_ssh_deploy_key()
    run('mkdir -p %(path)s' % env)
    if env.repo.startswith('git'):
        run('cd %(path)s; git clone -b %(branch)s %(repo)s .' % env)
        git.update()
    elif env.repo.startswith('ssh+svn'):
        run('cd %(path)s; svn co %(repo)s' % env)
    tool = django_exec().get("checkout_tool")
    if tool == "buildout":
        run('cd %(path)s; sh init.sh -c %(buildout_cfg)s' % env)
        django_utils.append_settings()
    elif tool == "virtualenv":
        import virtualenv
        virtualenv.create()
        django_utils.append_settings()
        django_utils.manage("syncdb --all --noinput")
        django_utils.manage("migrate --fake")
    else:
        print "WARNING: Couldnt find [checkout] tool - please set it to either virtualenv or buildout in your config.ini"
        print "Got tool: %s" % tool
        django_utils.append_settings()
    supervisor_update_config_file()
    nginx_update_config_file()

@task
def upload_ssl():
    """
    Upload the SSL key and certificate to the directories and with the filenames
    specified in your settings.
    """
    for site, site_dict in config.sites.items():
        ssl_key_path = prompt("SSL Key path (%s):" % site)
        ssl_cert_path = prompt("SSL Certificate path (%s):" % site)
        put(ssl_key_path, site_dict.get("deployment").get("ssl_key_path"))
        put(ssl_cert_path, site_dict.get("deployment").get("ssl_cert_path"))


import StringIO
from fabric.decorators import task
from fabric.api import run, env, put

from dploi_fabric.toolbox.template import render_template
from dploi_fabric.utils import config


@task(alias="reload")
def reload_nginx():
    run('sudo /etc/init.d/nginx reload')
    
@task
def update_config_file():
    output = ""
    template_name = 'templates/nginx/nginx.conf'
    for site, site_config in config.sites.items():
        context_dict = site_config
        context_dict.update({
            'domains': " ".join(site_config.deployment.get("domains")[site]),
            'www_processes': [site_config.processes[x] for x in site_config.processes if site_config.processes[x]["type"] == "gunicorn"],
        })

        output += render_template(template_name, context_dict)
    put(StringIO.StringIO(output), '%(path)s/../config/nginx.conf' % env)
    reload_nginx()



# -*- coding: utf-8 -*-
import os

from django.conf import settings
from django.template import Template
from django.template.context import Context

import dploi_fabric

settings.configure(DEBUG=True, TEMPLATE_DEBUG=True)

def render_template(path, context):
    path = os.path.join(os.path.dirname(dploi_fabric.__file__), path)
    if not isinstance(context, Context):
        context = Context(context)
    template_file = open(path)
    template = Template(template_file.read())
    return template.render(context)


import ConfigParser
import StringIO
from fabric.decorators import task
from fabric.api import env, get, put
import os
from fabric.contrib.files import exists
from .utils import config
import posixpath
from .utils import STATIC_COLLECTED, DATA_DIRECTORY
from pprint import pformat
from fabric.operations import run

def django_exec(dictionary = {}, tool="buildout"):
    # TODO: Remove this and change dependants to use utils.config
    config = ConfigParser.RawConfigParser()
    config_file = os.path.join(env.path, "config.ini")
    django_base = "." # default to current dir
    if exists(config_file):
        output = StringIO.StringIO()
        get(u"%s" % config_file, output)
        output.seek(0)
        config.readfp(output)
    
        try:
            tool = config.get("checkout", "tool")
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            tool = "buildout" # default to buildout

        try:
            django_base = config.get("django", "base")
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            pass
    if django_base == ".":
        django_base = ""
    if tool == "buildout":
        cmd = os.path.join(env.path, django_base, "bin/django")
        django_settings = os.path.join(env.path, django_base, "settings.py")
    else:
        cmd = "%s %s" % (os.path.join(env.path, "bin/python"), os.path.join(env.path, django_base, "manage.py"))
        django_settings = os.path.join(env.path, django_base, "settings.py")
    dictionary['django_exec'] = cmd
    dictionary['django_settings'] = django_settings
    dictionary['checkout_tool'] = tool
    return dictionary
    
def django_settings_file(dictionary = {}): # TODO: Remove this and change dependants to use utils.config
    return django_exec().get("django_settings")

@task
def manage(*args):
    """
    Proxy for manage.py
    """
    config.django_manage(" ".join(args))
    
@task
def collectstatic(staticdir='static'): # As defined in puppet config
    # TODO: Use utils.config
    run(('cd %(path)s; mkdir -p ' + staticdir) % env)
    manage("collectstatic", "--noinput", "--link")
    
@task
def append_settings():
    # TODO: make it work with multisites!
    append = config.sites["main"].get("django").get("append_settings", False)
    if append:
        site_config = config.sites["main"]
        settings_file_path = django_settings_file()
        print "Appending auto generated settings to", settings_file_path
        output = StringIO.StringIO()
        get(u"%s" % os.path.join(env.path, "../config/django.py"), output)
        output.seek(0)
        manual_settings = output.read()

        # START OF DIRTY DATABASE HACK


        additional_settings = """if "DATABASES" in locals():\n"""
        # DATABASES
        #additional_settings = "DATABASES = %s\n" % pformat(config.sites["main"].get("deployment").get("databases"))
        additional_settings +="    DATABASES = %s\n" % pformat(config.sites["main"].get("deployment").get("databases"))

        db_old_dict = config.sites["main"].get("deployment").get("databases")["default"]
        db_old_dict["ENGINE"] = db_old_dict["ENGINE"].replace("django.db.backends.", "")
        additional_settings += """else:
    DATABASE_ENGINE = "%(ENGINE)s"
    DATABASE_NAME = "%(NAME)s"
    DATABASE_USER = "%(USER)s"
    DATABASE_PASSWORD = "%(PASSWORD)s"
    DATABASE_HOST = "%(HOST)s"
""" % db_old_dict

        # // END OF DIRTY DATABASE HACK

        # CACHES
        processes = config.sites["main"].get("processes")
        cache_dict = {
            'default': {
                'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
                'LOCATION': 'unix:%s' % [processes[x] for x in processes if processes[x]["type"] == "memcached"][0].get("socket"),
            }
        }
        additional_settings += "CACHES = %s\n" % pformat(cache_dict)

        # PATHS
        additional_settings += """
STATIC_ROOT = "%(static_root)s"
MEDIA_ROOT = "%(media_root)s"
""" % {
            'static_root': posixpath.join(site_config.get("deployment").get("path"), STATIC_COLLECTED),
            'media_root': posixpath.join(site_config.get("deployment").get("path"), DATA_DIRECTORY, 'media/'),
        }

        output = StringIO.StringIO()
        get(settings_file_path, output)
        output.seek(0)
        settings_file = output.read()


        run("mkdir -p %s" % posixpath.join(site_config.get("deployment").get("path"), "_gen/"))
        put(StringIO.StringIO("%s\n%s\n%s" % (settings_file, additional_settings, manual_settings)), site_config.get("deployment").get("generated_settings_path"))
        put(StringIO.StringIO(""), posixpath.join(site_config.get("deployment").get("path"), "_gen/__init__.py"))


from fabric.operations import run, prompt
from fabric.api import task, env, get, put
from fabric.contrib.files import exists
import ConfigParser
import StringIO
import posixpath
from .utils import STATIC_COLLECTED, DATA_DIRECTORY
from django_utils import django_settings_file, append_settings
import os
from .messages import CAUTION
from .utils import config

@task
def update():
    test = run("cd %(path)s; git diff --stat" % env)
    if "files changed" in test:
        print CAUTION
        print "You have local file changes to the git repository on the server. Run 'fab %s git.reset' to remove them, " \
              "or keep them by applying the diff locally with the command 'git apply filename.diff' and upload it to your git host" % env.identifier
        print
        print "You now have the following options:"
        print
        print "[D]ownload diff"
        print "Continue and [R]eset changes"
        print "[E]xit"
        download_diff = prompt("What do you want to do?", default="D")
        if download_diff.lower() == "d":
            diff = run(("cd %(path)s; git diff --color .") % env)
            for i in range(1,50):
                print
            print diff
            for i in range(1,5):
                print
            exit()
        elif download_diff.lower() == "e":
            exit()
    run("cd %(path)s; find . -iname '*.pyc' -delete" % env)
    run("cd %(path)s; git fetch origin" % env)
    run("cd %(path)s; git reset --hard" % env)
    run("cd %(path)s; git checkout %(branch)s" % env)
    run("cd %(path)s; git pull origin %(branch)s" % env)
    if exists(posixpath.join(env.path, ".gitmodules")):
        run("cd %(path)s; git submodule init" % env)
        run("cd %(path)s; git submodule update" % env)
    append_settings()

@task
def diff(what=''):
    run(("cd %(path)s; git diff " + what) % env)

@task
def status():
    run("cd %(path)s; git status" % env)

@task
def reset():
    """
    discard all non-committed changes
    """
    run("cd %(path)s; find . -iname '*.pyc' -delete" % env)
    run("cd %(path)s; git reset --hard HEAD" % env)

@task
def incoming(remote='origin', branch=None):
    """
    Displays incoming commits 
    """
    if not branch:
        branch = env.branch
    run(("cd %(path)s; git fetch " + remote + " && git log --oneline .." + remote + '/' + branch) % env)
